#!/usr/bin/env python3
# Friday, 06/04/2021
#################################################################################
# setBrowser.py
# used in conjunction with the runBrowser shell script
#
# c2008-2019 by Paul Sherman <psherman2001@gmail.com>     
#									        
# This program is free software; you can redistribute it and/or			
# modify it under the terms of the GNU General Public License			
# as published by the Free Software Foundation; either version 3		
# 										
# This program is distributed in the hope that it will be useful,		
# but WITHOUT ANY WARRANTY; without even the implied warranty of		
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			
# GNU General Public License for more details.					
#										
# You should have received a copy of the GNU General Public License		
# along with this program; if not, write to the Free Software			
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.   
#################################################################################
# set the default browser on Absolute Linux
#********************************************************************************

import os, re, sys, subprocess, gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
cfile = os.path.join(os.path.expanduser("~/"), ".bashrc")
icefile = os.path.join(os.path.expanduser("~/"), ".icewm/toolbar")

brave_icon='prog "Web Browser" "brave" runBrowser'
chrome_icon='prog "Web Browser" "chrome" runBrowser'
chromium_icon='prog "Web Browser" "chromium" runBrowser'
falkon_icon='prog "Web Browser" "falkon" runBrowser'
firefox_icon='prog "Web Browser" "firefox" runBrowser'
palemoon_icon='prog "Web Browser" "palemoon" runBrowser'
vivaldi_icon='prog "Web Browser" "vivaldi" runBrowser'
waterfox_icon='prog "Web Browser" "waterfox" runBrowser'
ungoogled_icon='prog "Web Browser" "ungoogled-chrome" runBrowser'

class bchooser:

	def alert(self, message):
		box = Gtk.MessageDialog(None, 0, Gtk.MessageType.ERROR, Gtk.ButtonsType.OK, message)
		box.set_position(Gtk.WindowPosition.CENTER)
		box.set_transient_for(self.window)
		box.set_title('Error')
		box.run()
		box.destroy()	

	# This is a callback functionthe quits the program.
	def delete_event(self,widget,event,data = None):
		Gtk.main_quit()
		return False	


	def doClose(self, w=None):
		Gtk.main_quit()

	def changed_browser(self, evt=None, data=None):
		browser = self.combo.get_active_text()
		#
		# Set browser env. variable via user .bashrc ---------------------------------
		#
		whole = ""
		matched = ""
		browserline = 'export BROWSER=' + browser + '\n'
		file = open(cfile)
		while 1:
			line = file.readline()
			if not line:
				break
			if re.match('export BROWSER', line):
				line = browserline
				matched = "yes"
			elif line == "\n":
				continue
			whole = whole + line
		file.close()

		if not matched == "yes":
			whole = whole + browserline
			
		nfile = open(cfile, "w")
		nfile.write(whole)
		nfile.close()
		
		# update mime and desktop databases
		try:
			cmd1 = "update-mime-database /usr/share/mime 2>/dev/null"
			cmd2 = "update-desktop-database 2>/dev/null"
			print (browser)
			if "waterfox" in browser:
				newstr = waterfox_icon
			elif "falkon" in browser:
				newstr = falkon_icon
			elif "brave" in browser:
				newstr = brave_icon
			elif "firefox" in browser:
				newstr = firefox_icon
			elif "chrome" in browser:
				newstr = chrome_icon
			elif "vivaldi" in browser:
				newstr = vivaldi_icon
			elif "chromium" in browser:
				newstr = chromium_icon
			elif "palemoon" in browser:
				newstr = palemoon_icon
			elif "ungoogled-chrome" in browser:
				newstr = ungoogled_icon
			else:
				newstr = firefox_icon
			cmd4 = "killall -HUP icewm& > /dev/null 2>&1"
			os.system(cmd1)
			os.system(cmd2)
			subprocess.call(['sed', '-i', '/Browser/ c'+newstr, icefile])
			os.system(cmd4)
		except:
			pass

		os.system('zenity --width=220 --info --text="Default browser has been set." --timeout 3 2>/dev/null')
		#box = Gtk.MessageDialog(None, 0, Gtk.MessageType.INFO, Gtk.ButtonsType.OK, "Default browser has been set.\n")
		#box.set_position(Gtk.WindowPosition.CENTER)
		#box.set_transient_for(self.window)
		#box.set_title('Success')
		#box.run()
		#box.destroy()
		Gtk.main_quit()

	def __init__(self):		
		# set dropdown selections here
		brows = []
		os.chdir('/usr/share/applications')
		dFiles = os.listdir("/usr/share/applications")
		for df in dFiles: 
			try:
				f=open(df, "r")
				fileString = f.read()
				f.close()

				if re.search("WebBrowser", fileString):
					bname = os.path.splitext(df)[0]
					
					if (bname != "html2pdf") and (bname != "htmlpage") and (bname != "runBrowser") and (not "calibre" in bname) and (not "exo-web-browser" in bname):
						
						brows.append(bname.lower())
			except:
				pass

		# Create a new window.
		self.window = Gtk.Window()
		
		# Set the title of the window to "Combo Box".
		self.window.set_title("Web Browser")

		# Set a handler for delete_event that immediately exits Gtk.
		self.window.connect("delete_event", self.delete_event)

		# Set the border width of the window.
		self.window.set_border_width(16)

		self.window.set_position(Gtk.WindowPosition.CENTER)

		###########  layout  ################
		vbox = Gtk.VBox(homogeneous=False, spacing=16)
		self.window.add(vbox)
		
		bot_box = Gtk.HBox(homogeneous=True,spacing=0)
		btn_close = Gtk.Button(label="Cancel")
		btn_close.connect("clicked", self.doClose)
		bot_box.pack_start(btn_close, False, False, 0)
		
		btn_set = Gtk.Button(label=" Set ")
		btn_set.connect("clicked", self.changed_browser)
		bot_box.pack_start(btn_set, False, False, 0)
		
		lbl_about = Gtk.Label(label="Choose the Web browser\nyou would like to be the default")
		
		# Create a combo box.
		self.combo = Gtk.ComboBoxText()
		for l in brows:
			self.combo.append_text(l)
		self.combo.set_active(0)

		vbox.pack_start(self.combo, False, False, 0)
		vbox.pack_start(lbl_about, False, False, 0)
		vbox.pack_end(bot_box, False, False, 0)
		btn_close.show()
		btn_set.show()
		bot_box.show()	
		self.combo.show()
		lbl_about.show()
		vbox.show()
		self.window.show()
		
		if len(brows) < 2:
			os.system('zenity --width=220 --info --text="You only have one browser installed,\nso... not much to do here." --timeout 3 2>/dev/null')
			raise SystemExit		
		
			
	def main(self):
		Gtk.main()
		
if __name__ == "__main__" :
	widget = bchooser()
	widget.main()
