#!/usr/bin/env python3
#
# c2007-2019 by Paul Sherman 
# <psherman2001@gmail.com> 
# http://www.absolutelinux.org
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3	
# of the License, or (at your option) any later version.
#
from tkinter import *
import datetime, time, sys, os, fcntl, tempfile

icon = \
"""iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAH/npUWHRSYXcgcHJvZmlsZSB0eXBl
IGV4aWYAAHjarVhtluQoDvzPKfYIFiA+jgMI3psbzPE3JHCms7qqd3velLvKGIOQFFJIbjf//mu5
/+DHc0ouci6ppnThJ9ZYfcOgXPtn3+mK9td+/HmF549593rhMRVwD/sxzbO+YZ7fG3I88/1z3uVx
5JQj6Ly4BQY9WU8768oRFPyep/Ps6tnX4sOc8xvytuRe/PU5ZjhDGJPBOz8DhQt/daMP0CDU0Oze
8MxeR4wx20wM/L3v3Gv4xXmv0RffXe3Mh09XuCudBemLj8488fe+Mw89NaL3yR8v5noFwS++W0vK
WnNb12KCp5I7Rt2m2AgLO1wZbFvClfHLGGe7Kq4CEwecLkCz4xqOKnl4e1EkoUaLpt0HDagY/fQZ
d++HDzZXQvbVDwMj6kXLZ8AjLhRgM4BawLR/6UJ2brXzBhWcLISVniCMsOOXy303+U+ul6C1NHSJ
rvLyFfTyGoBQQ5HTv1gFQGgdn7L51y73iJvrAWwAgmxuLjCwXX2L6Ezv2AqGc8A6vqK7drhTliMA
LsLZDGUoAIErIbAp0ZW9z0TwYwE+DZr7EH0HAsTshdwCNiEkgFO8no09mWytZ7+nQS2aGiGFDGiQ
QAArRkb85FgQQ40DR8fMiTMXrtxSSDGBpFJOylEthxwz55RzLrnmVkKJhUsquZRSS6u+BlAY11Sz
q6XW2hoObRDdsLthRWvd99Bj55567qXX3gbCZ8TBI408yqijiZcgSH9Jkp0UqdImTYTSjJNnmnmW
WWdbiLUVVly80sqrrLraC7WD6idq9AW536NGBzVFLNq6/EYN0znfIkjphBUzIOYjAfGsCCCgvWJ2
FYrRK3KK2VU9koI9UCNWcIQUMSAYJ3le9MLujdxvcXMc/wg3/xNyTqH7N5BzCt1B7lfcvkFNmlWU
YABpFqpPr7BAbFjUfME/8LHvV+swC+naJ2ragnJ9JhS3a0K/KaGtcmHQF6ZWvHTnZOx1ofXma4Wy
KA+5kGQ8UNQHWcRBcvGcu4hWIQarccN+apXnJTxn4HXpakdQfHIe8P/eDIqITFw6lndAYULphzvK
6xkiIHmxCFxXIZJEywfk9oxpqDO6VkPOUrGOOvxZ9YSZmVfQR6i4AgsE5R4l9wwTFu5e9Jh4a5KF
MsvirGYgEJEdIBxTAsOMHUM1wcCZcjjz9XJAnwtIZTXYegboQ6pAk3s78cwqObx3ureI15zuLAJL
BlS/1BIJEAr/rtwxoHCsxXFUePvVqcEeuwIMzMQjqG0dIrpujlvvp1dhrjdzExbVZO/GArHRGy1e
dtJaNtjYGBRY0C71ByBIFQOuD3d0Pc0pMlfQc3+L7kOfRgyT96uXCtOlzsszPxyuSmle+MFv524P
4ilpqCDhnztoTnG8YvwQz4ykuOpJmfK8hzba6nH2yZo9vT2WuHvN/GXbn93d/78hiKQ+S2uoYG1l
pQF7se/OBprRmOtSkfiI/NaYp7BZWwoavy492UJiEBaQM8mDNd2niOUaqE9sZdsrPcgNebxXhlkh
fmwh12xgy54twKdHXHjJjeyxIPtBxtmEDGkjtdmmbQs4uQocK34L9ThN0vdWT2gEegHzhavNNXVq
vFeNifQyJtMXfahCAwlm+FbkvCaMJe3FYMi0hFPr4YQzsnEx+E1za46gsYNmDuFzx6XRTFgWVF0/
QPA72e24wt4i3mK5LhChNOFhqVEZbmnYynJHWuRn3KEKRdCze1KIBjUYa8CHaZ8Y39F+E0+ulyVq
lzt5kH9D+agjSdKZuZMWdKOCghij3LKQ64zagvZ1FwqrNFYZGlIkbSDustGB96SuuKFmdfFwMgGv
mXRzQqQYpiiefVQEqKYssFrOZrXATAE+cS/rALC9QhUqo+g1vek7397x7PtYGsPgbMyuWibKq+rA
9jght+RnCgBBlhMIUAAluc+s+mvIWLy4Z1ihuC89ZVr0aHIHqQjJqtIzBu36TLCtdEh47fzLzlIt
7gqKKZdTUO3NbWBHP2Y3NZBe8roSx3JQzkaPk6BRVNU0PdRi2H1eIbpQV8GPjIEv8iZMct/WT3zw
oX5pXKIMkRF8R7BLqEanGXzyZFS9W0AmtJPtHZNAXavnCZ4TodmSJYV3hKJPk7CZFt8xDgmbwk7v
lrpw+5LWJ6MRUc1yGu3Zt7zngJVXiOBzUY93Xa4ENTvdLUy0FqZr1IBlNJIXN31Cp8O6HvPunDzQ
/0xkPCgdItmb03Uf5uVKRiPzpcOqQ0NtvSILGp03aJxOisS5y8WjKCi1ArC5uUPLU/ysX7grjdDQ
epfu5F7GJspfgBnfB9XSNyjXRLQEZlkF+JC/IB9vtUA5ZAUoVtSW5TW1I1yO2BonGyKqBljXlLPw
fJl3IvZNtRqApZlDLT8mgVLRxT3iqopybLfCrhX+VYAtMFitc9+Y+7/uaVMpWvBHHDsjzILSMr4Q
aB6gWZL9Wh8LR3OXUrq2gUegjKpv3WFK/f8czZ+qHu5cTXt8w7yanNy7CaUf+lL3Y8N6E2+AoHDt
CXyyoApoE1Q+qHlXWjNoM3XYTH1pk4isngDcagy+EXKzPrUU7XEwhRS01rTac1Lyb7wtx9pHsVB3
bll8d69W4nb3/tlmaa3Dt8j6oSFHMGqLj2xC6UKMyY2aGfMVUPdnyH900ui4sz/5QK7elr6aOJaq
zfDaqsMZQ0s8kvADCcFuSaz9xzNpT4YufEZVtHH/BWCtAJOyeVzzAAAABmJLR0QA/wD/AP+gvaeT
AAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4wEZFDMt5u7AwAAABeRJREFUWMO9l19M01kW
xz+/X1va7IhdMxBnYvatMoN2A5v1DxoSIj5MYnQGfZmZxvgn6rqEWRNfKvPgPiiio4nrE/pUQ6PB
REmAEWNsoL4YxJEMKFgazYYqDbGKDMjSQtvf2Qd//U1/0BJ0MnOSk9x7v+fe+73nnt+554eIoKsq
In/TNO0nQHO73ZqI/CwibhGxikixiFzZsWNHGtASicTPIvJ3HXOKyA+NjY2zgDx79iwmIjt0jMU0
01CeP3/+FxH5j8/n+x8ggASDwXEROTE4OPh1KpX6rre391kGa2homBaRM4ODg9+8evWqLh6PD2Qw
ICkiTdFodJ2IOBYlAFiARiAeCATmrly5omUWun//fvrMmTMpQGpra1PDw8PJDHbu3Dmtt7d3NtMX
kbksApJIJBJAGmg7e/asMx8BFfgn8D3gcDgcNovFoqCLzWZTly1bZgFwOp0Wm81mzcIUu91ewK9i
y2pjtVrtgAp85fV6LwSDQXKJCtRmOhaLBavV+iuoqqiqCoCiKEY7Y6soCkuUbz0eT1E+0HDrqVOn
ZGZmRrLcKhUVFaZ+ph2LxeTSpUs5sUAgII8fP5bsK1EUZWO+GDAZ9vf3S0Y6OztNWEtLi4E9ffrU
hHm9XgPLPkRG3W531ZII/F66c+fO6nxB+IeIy+UqAJRcQfiHSGFhYUGucSPk/X4/b9++JZlMoqqq
EeWappFOp9E0DVVVURQFEUHTNOMOVVXFbrezevVqXC4XK1asIJVKMTY2RiQSob6+ftbhcFjzkZND
hw7J7yiaiDwXkWo93S+MgVgsZmI0MTFBV1cXwWCQBw8eGOPxeJy2tjZu3LhBNBo1zQmFQhw8eBBF
UUy6Z88e5dGjR8uAj/NduVRVVZkoz4/gkZER0/j+/fsN25GREZNte3u7tLW1yZMnT6SjoyMb+y/w
5fT0tGXBZ7h58+ZFCYTDYWN806ZNhl1ra6thMzMz887fmiaA3Lx507BLp9Oax+NJAnPAv8fHxx0m
Ahs3bjQRaG5uNhbOxrq7u412IBAQQHw+n2nu6OioANLY2LggFoLBYBJIAQ0i8ieDwPr1698rqqam
pgSQ8+fPL8BCoZAAcvjw4ZwBeevWrTQw29LSclhECnIS6OrqkgMHDkhtba3U19cvWCU7/8+XgYEB
AWTv3r15D+ByuTRgSEQ+tQIUFJhzxNatW0393bt3s3btWuNLAOjv71808WS/qvOlp6eH4uLiz5ua
mjZbl5LFsp/h27dvA1BWVpbTNvNEj42N5V2vqKhIAairq/vCyjtfmgzC4TB3797FYrFQWFhIaWmp
gTU3Ny8p9S7mAYCTJ09y/Pjxz3ISKCkpoaSkJOfE9vZ26urq8i6saRoAHR0ddHZ2Eo/HmZubw263
s3LlSiorKwFYs2YNQJE1U91ky9GjR7lw4YJp0ezqJ7NJzqyWdZjt27cvwD0eD1evXkVRFAGSVr2+
Mxllbw4wNDSE2+0GYN++fVy8eJGmpqacBMrLy+nu7ubNmzeMj4+TSCSYmpri9evXxONxTp8+DSAD
AwOanh2Rbdu2LZoJI5GIgd25cyfvJ/gektYT0vfopfMCi+HhYQmFQvLixQtzJtFT7b179z5499HR
0SQwC5QDPAGkp6dHy3o+M5otKRGZFpGJXbt2Tf8GL2j66W9Go1EbgEcfEEAqKyu1ioqKdHV1tbZl
yxZtw4YNsmrVKgFS4XD4koj84+XLl98Bvxw7dkx7390vX76cBiadTuf6zK+ZAnwLDOluWay47AoG
gx+JiKOmpuYgEG9oaEiLSHopJ/f7/Wn9RawTEYvp31BE7D6fz1lTU/NnwFlaWuo8cuTICr/f//HD
hw8/aW1tLQaWA4o+pwDYC0wCqUgkkhCRWRFJ6oQymorFYik91maAf4lIwfyf0w9VS1lZWTlwS/de
wuv1Tly7du2X69evT544cWJaP3ES6Kl6V/mYChJlfhb8EOnr67OtW7fur0ANUAF8qpdfL4E+4MfJ
ycm+5cuXz+hXacj/AQJ7EhzbywKoAAAAAElFTkSuQmCC"""

lockfile = os.path.normpath(os.getenv("HOME") + '/' + os.path.basename(__file__) + '.lock')
if os.path.exists(lockfile):
	 sys.exit()
	 
def callback():
	try:
		os.remove(lockfile)
	except:
		pass
	root.destroy()

def maxdays(mon,year):
		
	if mon==4 or mon==6 or mon==9 or mon==11 :
		return 30
	
	if mon==2 :
		if year%4==0 and (year%100!=0 or year%400==0): return 29
		return 28
	else :
		return 31    
			

def setTheDate():
	#mth, dy, yr, hr, min
	
	mRange=test.listYear.yview()
	mSpot = int(((mRange[0] + mRange[1]) / 2) * 96)
	yr = test.listYear.get(mSpot)
	
	mRange=test.listDay.yview()
	mSpot = int(((mRange[0] + mRange[1]) / 2) * 31)
	dy = test.listDay.get(mSpot)
	
	mRange=test.listMonth.yview()
	mSpot = int(((mRange[0] + mRange[1]) / 2) * 12)
	mth = test.listMonth.get(mSpot)
	
	if dy > maxdays(mth, yr):
		os.system('zenity --width=220 --info --text="Error entering date" --timeout 3 2>/dev/null')
		return 0
	else:
		pass
		
	mRange=test.listHours.yview()
	mSpot = int(((mRange[0] + mRange[1]) / 2) * 13)
	intHr = int(test.listHours.get(mSpot)) + int(radVar.get())
	#ensures a switch from am to pm does not put us into 24:15 land...
	if intHr > 23:
		intHr = intHr - 12
	hr = intHr
	
	mRange=test.listMinutes.yview()
	mSpot = int(((mRange[0] + mRange[1]) / 2) * 60)
	min = test.listMinutes.get(mSpot)
	
	strDate = "%02i/%02i/%4i %02i:%02i:00" % (mth, dy, yr,hr,min)
	bareDate = "%02i/%02i/%4i" % (mth, dy, yr)
	bareTime = "%02i:%02i:00" % (hr,min)
	
	cmdDate="/usr/bin/date -s \"" + strDate + "\""
	cmdRTC="/sbin/hwclock --systohc"

	try:
		status = os.system(cmdDate)
		status = os.system(cmdRTC)
			
		os.system('zenity --width=220 --info --text="Desktop and RTC clock set" --timeout 3 2>/dev/null')
		
	except:
		pass
		
	os.remove(lockfile)
	root.destroy()

def syncOnline():

	if not os.path.exists("/usr/sbin/ntpdate"):
		os.system('zenity --width=320 --info --text="The required ntpdate utility seems to be missing." --timeout 3 2>/dev/null')	
		
		return 0
	cmdSync='/usr/sbin/ntpdate -sb time.nist.gov | zenity --progress --width=300 --auto-close --title="Attempting to sync time..." 2>/dev/null'
	try: 
		status = os.system(cmdSync)

	except:
		os.system('zenity --width=340 --info --text="There was an error attempting to sync clock to NTP time server." --timeout 3 2>/dev/null')	
		
	os.system('zenity --width=220 --info --text="Time has been synced!" --timeout 3 2>/dev/null')	
	os.remove(lockfile)
	root.destroy()

			
	
class Test(Frame):
		
	def createWidgets(self):
	
		self.lblHeader = Label(self, text="Adjusts System Clock and the RTC")
		self.lblHeader.pack(padx=10, pady=6)
	
		frameDate = Frame(self, relief=RAISED, borderwidth=1, background="#cccccc")
		frameDate.pack(padx=12)
		
		#Widgets for month
		fmonth = Frame(frameDate, relief=RIDGE, borderwidth=2)
		fmonth.grid(row=0, column=0, padx=6, pady=6)
		
		self.lblMonth = Label(fmonth, text="month")
		self.lblMonth.grid(row=0, column=0)
		
		self.listMonth = Listbox(fmonth, height=1, width=6)
		self.listMonth.grid(row=1, column=0)
		
		self.scrollMonth = Scrollbar(fmonth)
		self.scrollMonth.grid(rowspan=2, row=0, column=1)

		#Range and bindings fot month
		for i in range(12, 0, -1):
			self.listMonth.insert(END, i)
			self.listMonth.yview(abs(int(now.month)-12))
			
		self.listMonth.config(yscrollcommand=self.scrollMonth.set)
		self.scrollMonth.config(command=self.listMonth.yview)
		
		#Widgets for day
		frameDay = Frame(frameDate, relief=RIDGE, borderwidth=2)
		frameDay.grid(row=0, column=1, padx=6, pady=6)
		
		self.lblDay = Label(frameDay, text="day")
		self.lblDay.grid(row=0, column=0)
		
		self.listDay = Listbox(frameDay, height=1, width=6)
		self.listDay.grid(row=1, column=0)
		
		self.scrollDay = Scrollbar(frameDay)
		self.scrollDay.grid(rowspan=2, row=0, column=1)

		#Range and Bindings for Minutes
		for j in range(31, 0, -1):
			self.listDay.insert(END, j)
			self.listDay.yview(abs(int(now.day)-31))
			
		self.listDay.config(yscrollcommand=self.scrollDay.set)
		self.scrollDay.config(command=self.listDay.yview)
		
		#Widgets for year
		frameYear = Frame(frameDate, relief=RIDGE, borderwidth=2)
		frameYear.grid(row=0, column=2, padx=6, pady=6)
		
		self.lblYear = Label(frameYear, text="year")
		self.lblYear.grid(row=0, column=0)
		
		self.listYear = Listbox(frameYear, height=1, width=6)
		self.listYear.grid(row=1, column=0)
		
		self.scrollYear = Scrollbar(frameYear)
		self.scrollYear.grid(rowspan=2, row=0, column=1)

		#Range and Bindings for Minutes
		for j in range(2099, 2003, -1):
			self.listYear.insert(END, j)
			self.listYear.yview(abs(int(now.year)-2099))
			
		self.listYear.config(yscrollcommand=self.scrollYear.set)
		self.scrollYear.config(command=self.listYear.yview)
		
		fSettings = Frame(self, relief=SUNKEN, borderwidth=1, background="#cccccc")
		fSettings.pack(pady=12)
		
		#Widgets for Hours
		fh = Frame(fSettings, relief=RIDGE, borderwidth=2)
		fh.grid(row=0, column=0, padx=6, pady=6)
		
		self.lblHours = Label(fh, text="hour")
		self.lblHours.grid(row=0, column=0)
		
		self.listHours = Listbox(fh, height=1, width=6)
		self.listHours.grid(row=1, column=0)
		
		self.scrollHours = Scrollbar(fh)
		self.scrollHours.grid(rowspan=2, row=0, column=1)

		#Range and bindings fot Hours
		for i in range(12, -1, -1):
			self.listHours.insert(END, i)
			self.listHours.yview(abs(intHour-12))
			
		self.listHours.config(yscrollcommand=self.scrollHours.set)
		self.scrollHours.config(command=self.listHours.yview)
		
		#Widgets for Minutes
		fm = Frame(fSettings, relief=RIDGE, borderwidth=2)
		fm.grid(row=0, column=1, padx=6, pady=6)
		
		self.lblMinutes = Label(fm, text="minute")
		self.lblMinutes.grid(row=0, column=0)
		
		self.listMinutes = Listbox(fm, height=1, width=6)
		self.listMinutes.grid(row=1, column=0)
		
		self.scrollMinutes = Scrollbar(fm)
		self.scrollMinutes.grid(rowspan=2, row=0, column=1)

		#Range and Bindings for Minutes
		for j in range(59, -1, -1):
			self.listMinutes.insert(END, j)
			self.listMinutes.yview(abs(int(now.minute)-59))
			
		self.listMinutes.config(yscrollcommand=self.scrollMinutes.set)
		self.scrollMinutes.config(command=self.listMinutes.yview)
		
		# Radio Buttons for AM/PM
		fAmPm = Frame(fSettings, relief=RIDGE, borderwidth=2)
		fAmPm.grid(row=0, column=2, padx=6, pady=6)
		
		self.radioAmPm = Radiobutton(fAmPm, text="am", variable=radVar, value="0")
		self.radioAmPm.grid(row=0, column=0)
		self.radioAmPm = Radiobutton(fAmPm, text="pm", variable=radVar, value="12")
		self.radioAmPm.grid(row=1, column=0)
		
		btnFrame = Frame(self)
		btnFrame.pack(fill=X, pady=2)
		bCancel = Button(btnFrame, text="Close", padx=12, command=callback).grid(row=0, column=0, padx=10)
		bApply = Button(btnFrame, text="Apply", padx=12, command=setTheDate).grid(row=0, column=1, padx=10)
		bNTPdate = Button(btnFrame, text="SyncOnline", padx=12, command=syncOnline).grid(row=0, column=2, padx=10)
		
	def __init__(self, master=None):
		Frame.__init__(self, master)
		Pack.config(self)
		self.createWidgets()
		

root = Tk()
root.protocol("WM_DELETE_WINDOW", callback)

fp = open(lockfile, 'w')
try:
	fcntl.lockf(fp, fcntl.LOCK_EX | fcntl.LOCK_NB)
except IOError:
	callback
	
now = datetime.datetime.now()
radVar = StringVar()
strYear = repr(now.year)
strMonth = repr(now.month)
strDay = repr(now.day)
strYear = repr(now.minute)
tmpHour = int(now.hour)
if tmpHour >= 12:
	intHour = abs(int(now.hour) - 12)
	radVar.set("12")
else:
	intHour = int(now.hour)
	radVar.set("0")
	
test = Test()
	
# routine to center root window  on screen
scrX, scrY = root.winfo_screenwidth(), root.winfo_screenheight()
wStart = (scrX - 300) / 2
hStart = (scrY - 220) / 2
root.geometry("%dx%d%+d%+d" % (300, 220, wStart, hStart))
root.title("Set System Time")

img = PhotoImage(data=icon)
root.tk.call('wm', 'iconphoto', root._w, img)

#test for valid date
maxDate = maxdays(2,2004)

###############
test.mainloop()
