#!/usr/bin/perl
############################################################################
# addremove.pl
# Tuesday, 01/25/2022
# c2009-2022 by Paul Sherman                                                    
# <psherman2001@gmail.com>                          
#									        
# This program is free software; you can redistribute it and/or			
# modify it under the terms of the GNU General Public License			
# as published by the Free Software Foundation; either version 3		
# of the License, or (at your option) any later version.			
#										
# This program is distributed in the hope that it will be useful,		
# but WITHOUT ANY WARRANTY; without even the implied warranty of		
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			
# GNU General Public License for more details.					
#										
# You should have received a copy of the GNU General Public License		
# along with this program; if not, write to the Free Software			
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.   
#############################################################################
# based upon code by Paul Malcher, called "Slackware GUI Packager Manager"
# I adapted code to include txz packages, avoid erroring out,
# UI improvements and refuses to install packages already installed.
# written for Absolute Linux, requires perl-TK
#############################################################################
use Tk;
use Tk::DialogBox;
use Tk::TextUndo;

$ENV{'PATH'} = '/bin:/usr/bin:/sbin';

my $my_icon_data = <<'EOF';
/* XPM */
static char * addremove_xpm[] = {
"24 24 190 2",
"  	c None",
". 	c #E42234",
"+ 	c #E73D55",
"@ 	c #FEF6F9",
"# 	c #ED6780",
"$ 	c #E42E44",
"% 	c #FEEEF4",
"& 	c #FBB7C4",
"* 	c #FDD9E5",
"= 	c #E53546",
"- 	c #DD1D2A",
"; 	c #FCD2DA",
"> 	c #FBBFCF",
", 	c #F6829B",
"' 	c #F6849F",
") 	c #FCC8D4",
"! 	c #DF3142",
"~ 	c #DB1B24",
"{ 	c #FCD0DC",
"] 	c #F67E98",
"^ 	c #F68099",
"/ 	c #F57D97",
"( 	c #FCCAD7",
"_ 	c #DD313D",
": 	c #D81821",
"< 	c #FCCEDA",
"[ 	c #FCCCD9",
"} 	c #F57B95",
"| 	c #F47A94",
"1 	c #F4728B",
"2 	c #F57792",
"3 	c #FBB7C6",
"4 	c #D51F1B",
"5 	c #D91B24",
"6 	c #FCD2DD",
"7 	c #FCCEDC",
"8 	c #F4748F",
"9 	c #F27088",
"0 	c #F26A84",
"a 	c #F0647F",
"b 	c #F16784",
"c 	c #F6A6B9",
"d 	c #CF2219",
"e 	c #D41914",
"f 	c #FCCAD8",
"g 	c #FDD7E5",
"h 	c #F77D9A",
"i 	c #F47691",
"j 	c #F4748C",
"k 	c #F06680",
"l 	c #EF617C",
"m 	c #EF5F77",
"n 	c #EF5972",
"o 	c #F15B77",
"p 	c #F8B2C6",
"q 	c #CE2918",
"r 	c #D1251D",
"s 	c #DD333B",
"t 	c #ED798C",
"u 	c #E3374B",
"v 	c #E02E3E",
"w 	c #FDD3DC",
"x 	c #F58197",
"y 	c #EF6781",
"z 	c #EF637D",
"A 	c #EE6078",
"B 	c #EE5A73",
"C 	c #EE566F",
"D 	c #DA262B",
"E 	c #DA262E",
"F 	c #DE3844",
"G 	c #D62C2C",
"H 	c #C42F16",
"I 	c #C4240A",
"J 	c #FEECF3",
"K 	c #F47E94",
"L 	c #EE5C74",
"M 	c #EC546A",
"N 	c #EC506A",
"O 	c #EF4D68",
"P 	c #BD2E0D",
"Q 	c #C2280A",
"R 	c #FEE8F1",
"S 	c #F2748F",
"T 	c #EB4F69",
"U 	c #EA4E68",
"V 	c #EA4A60",
"W 	c #EB475E",
"X 	c #EE4460",
"Y 	c #BD3511",
"Z 	c #C0280A",
"` 	c #FDE5EF",
" .	c #F06A80",
"..	c #EA465D",
"+.	c #E93F53",
"@.	c #E83C54",
"#.	c #EC3A53",
"$.	c #B92F0F",
"%.	c #149C5D",
"&.	c #D0F5D0",
"*.	c #60D47D",
"=.	c #41C769",
"-.	c #42C76C",
";.	c #47B977",
">.	c #48B679",
",.	c #45B978",
"'.	c #2B7A6D",
").	c #08AA58",
"!.	c #C6FBBD",
"~.	c #42EA52",
"{.	c #23E33E",
"].	c #21E341",
"^.	c #1EE23F",
"/.	c #1FDF43",
"(.	c #1CE03C",
"_.	c #0FA15B",
":.	c #08A05A",
"<.	c #BEFBB9",
"[.	c #38E849",
"}.	c #20DA3F",
"|.	c #21D543",
"1.	c #1FD341",
"2.	c #1BD33E",
"3.	c #0E9C5B",
"4.	c #12AC57",
"5.	c #19B959",
"6.	c #24CA4F",
"7.	c #1EC651",
"8.	c #15C149",
"9.	c #8CF28A",
"0.	c #2CE441",
"a.	c #20D442",
"b.	c #1CCE3D",
"c.	c #17CF3E",
"d.	c #13CF3B",
"e.	c #0EB64D",
"f.	c #0EB852",
"g.	c #0DAF55",
"h.	c #0E9E59",
"i.	c #0EA458",
"j.	c #23E343",
"k.	c #29EB3B",
"l.	c #20E63C",
"m.	c #1EDA41",
"n.	c #20D041",
"o.	c #1DCF43",
"p.	c #19CD3F",
"q.	c #15CB40",
"r.	c #11CB3D",
"s.	c #0FCB40",
"t.	c #0CCE3E",
"u.	c #0ACE3D",
"v.	c #0AC63F",
"w.	c #0C9C57",
"x.	c #0DA357",
"y.	c #1EDE42",
"z.	c #1DE13D",
"A.	c #1FD141",
"B.	c #1ACE40",
"C.	c #17CB41",
"D.	c #12CC3E",
"E.	c #0CCA3D",
"F.	c #0AC840",
"G.	c #0ACA40",
"H.	c #09C346",
"I.	c #0A9652",
"J.	c #0C9E5B",
"K.	c #1BCF45",
"L.	c #18D23B",
"M.	c #13CD3A",
"N.	c #10CC41",
"O.	c #0ECA3F",
"P.	c #0BC740",
"Q.	c #0BC745",
"R.	c #0B9355",
"S.	c #0A9857",
"T.	c #11CB45",
"U.	c #10D03D",
"V.	c #0CCC42",
"W.	c #0ACC41",
"X.	c #099954",
"Y.	c #09935A",
"Z.	c #09C543",
"`.	c #079755",
" +	c #099357",
".+	c #089454",
"++	c #079752",
"@+	c #0AD03D",
"#+	c #09A757",
"$+	c #078752",
"                      .                         ",
"                    + @ #                       ",
"                  $ % & * =                     ",
"                - ; > , ' ) !                   ",
"              ~ ; { ] ^ / ] ( _                 ",
"            : < [ ^ / } | 1 2 3 4               ",
"          5 6 7 / } | 8 9 0 a b c d             ",
"        e f g h i j 9 0 k l m n o p q           ",
"      r s t u v w x y z A B C D E F G H         ",
"              I J K L B M N O P                 ",
"              Q R S T U V W X Y                 ",
"              Z `  .W ..+.@.#.$.                ",
"              %.&.*.=.-.;.>.,.'.                ",
"              ).!.~.{.].^./.(._.                ",
"              :.<.[.^.}.|.1.2.3.                ",
"      4.5.6.7.8.9.0.a.1.b.c.d.e.e.f.g.h.        ",
"        i.j.k.l.m.n.o.p.q.r.s.t.u.v.w.          ",
"          x.y.z.A.B.C.D.s.E.F.G.H.I.            ",
"            J.K.L.M.N.O.P.v.G.Q.R.              ",
"              S.T.U.V.v.v.G.W.X.                ",
"                Y.Z.G.F.G.G.`.                  ",
"                   +W.u.u..+                    ",
"                    ++@+#+                      ",
"                      $+                        "};
EOF


$main = MainWindow->new(-title=> "Add/Remove Packages", -borderwidth=> 8);
$menubar = $main->Frame()->pack(-side => "top", -fill => "x");
$main->resizable(0,1);
$main->geometry("640x600+0+0");
$main->attributes('-zoomed', True);
$main->minsize(600, 650);

$main->Pixmap( 'my_icon', -data => $my_icon_data );
$main->Icon( -image => 'my_icon' );


# define the contents and variable of some dialog boxes
$about = $main->DialogBox(-title=>"About",-buttons=>["OK"]); #Creates the about Dialog
$aabout = $about->add("Label",-text=>"original source by Paul Malcher\nAdapted for Absolute by Paul Sherman\n\nuses perl-tk as a front-end\nfor \"installpkg\" and \"removepkg\"\nand just generally make listing and \nviewing package info a bit easier.\n\nVersion-1.4")->pack;

$ahelp = $main->DialogBox(-title=>"Add/Remove Help",-buttons=>["OK"]); #Creates the help Dialog
$aahelp = $ahelp->add("Label",-justify=>"left",-text=>"\n\"Install a Package\"\n  lets you browse your system\n  for TXZ and TGZ packages to install.\n\n\"Remove Package\"\n  Will remove the currently\n  selected package from your system.\n\n\Double-Click\n  On a selected package\n  to view the package information.\n")->pack;


# File menu
$filemenu = $menubar->Menubutton(-text => 'File', -underline => 0,-tearoff => 0)->pack(-side=>'left');
$filemenu->command(-label => 'Exit',-command => sub{exit});

$aboutmenu = $menubar->Menubutton(-text => 'Help', -underline => 0,-tearoff => 0)->pack(-side=>'left');
$aboutmenu->command(-label => 'About',-command => sub{$about->Show;});
$aboutmenu->command(-label => 'Help',-command => sub{$ahelp->Show;});

# defines the gui layout
$code_font = $main->fontCreate('code', -family => 'sans', -size => 14);
$pkglistl = $main->Label(-text=>"Installed Package List \nSelect package to remove...\n", -font=>'code')->pack();
$pkglistl2 = $main->Label(-text=>"Double Click on a package to see a discription\nRight-Click to see list of files\nOr choose \"Install a Package\" to browse for an installer.\n")->pack();
$pkglist = $main->Listbox(-relief => 'flat', -fg=> 'gray12', -background=>'gray79', -selectforeground=> 'RoyalBlue', -selectbackground=> 'seashell2', -height=>20, -borderwidth=>12, -font=> "Sans 18 bold");
$scroll = $pkglist->Scrollbar(-command => ['yview', $pkglist], -width=>18);
$pkglist->configure(-yscrollcommand => ['set', $scroll]);
$pkglist->pack(-fill => 'both', -ipady => 100, -expand => 1);
$scroll->pack(-side=>'right',-fill => 'both');
$pkginfol = $main->Label(-text=>'Package Info')->pack();
$pkginfo = $main->Scrolled(TextUndo,-height => 16, -scrollbars=>'osoe',-background=>'white', -wrap => 'none',-state=>'disabled')->pack(-fill=>'both',-expand=>0);


# buttons
$pkglb = $main->Button(-text=>'Exit',-command => sub{exit})->pack(-side=>'right');
$pkgrmb = $main->Button(-text=>'Remove Package',-command => sub{
$sfile = $pkglist->get(active);
$cdel = $main->DialogBox(-title=>"Package Removal",-buttons=>["Remove","Cancel"]);
$cdel->add("Label",-text=>"\nWarning you are about to unistall:\n\n$sfile\n\n     Are you sure you want to do this?      \n")->pack;
$result = $cdel->Show;


if($result eq "Remove"){

	$label->configure(-text=>'Removing... please wait', -foreground => 'red');
	$label->update;
	system("/sbin/removepkg $sfile");
	$label->configure(-text=>"$sfile REMOVED", -foreground => 'black');
	$label->update;
	&pkglistr();
	$main->geometry("640x600+0+0");
}else{
	$label->configure(-text=>'Unistall Aborted!', -foreground => 'black');
	$label->update;
	&pkglistr();
	}
})->pack(-side=>'right');

$pkgaddb = $main->Button(-text=>'Install a Package',-command => sub{
	if($open eq ""){
	my $types = [['Package TXZ', '*.txz'],['Package older TGZ', '*.tgz']];
	my $open = $main->getOpenFile(-filetypes=>$types);
	$longname = substr($open, 0, length($open)-4);
	@fArray = split(/\//, $longname);
	$fName = $fArray[-1];
	$curname = "/var/log/packages/" . $fName;
	if (-e $curname){
		$err = $main->DialogBox(-title=>"Error",-buttons=>["Ok"]);
		$err->add("Label",-text=>"Error: That package is already installed")->pack;
		$err->Show;
	} else {
	$label->configure(-text=>'Installing... please wait', -foreground => 'red');
	$label->update;
	system("installpkg $open");
	$label->configure(-text=>"$fName INSTALLED", -foreground => 'black');
	$label->update;
	&pkglistr();
	$pkglist->update;
	# send us to index of newly installed package
	@items = $pkglist->get(0,'end');
	$id = 0;
	$nIndex = 0;
	foreach (@items) {
		if ($_ eq $fName){
			$nIndex = $id;
			last;
		} else {
			$id++;
		}
    }
	$pkglist->yview($nIndex);
	}}
})->pack(-side=>'right');

$label = $main->Label(-text=>'')->pack(-side=>'bottom');

$pkglist->bind('<Double-1>' , \&loaddesc);
&pkglistr();

$pkglist->bind('<Button-3>' , \&showfilelist);
&pkglistr();

$pkglist->bind('<<ListboxSelect>>', \&onselect);

sub onselect{
	$pkginfo->configure(-state=>'normal');
	$pkginfo->delete('1.0','end');
}
	
sub showfilelist{
	$pf = $pkglist->get(active);
	$pkginfo->configure(-state=>'normal');
	$pkginfo->delete('1.0','end');
	$flag = 0;
	open(desc,"</var/lib/pkgtools/packages/$pf");
	@lines = <desc>;
	close desc;
	# make list case-insensitive, to make finding packages easier
	@lines = sort { "\L$a" cmp "\L$b" } @lines;
	foreach(@lines){
	if($_ eq "FILE LIST\:\n"){
	$flag = 1;
	}
	if($flag ne "0"){
	$pkginfo->insert('end',"$_");
	}
	}
	$pkginfo->configure(-state=>'disabled');
}

sub loaddesc {
	$pf = $pkglist->get(active);
	$pkginfo->configure(-state=>'normal');
	$pkginfo->delete('1.0','end');
	$flag = 0;
	open(desc,"</var/lib/pkgtools/packages/$pf");
	@lines = <desc>;
	close desc;
	foreach(@lines){
	if($_ eq "FILE LIST\:\n"){
	$flag = 1;
	}
	if($flag ne "1"){
	$pkginfo->insert('end',"$_");
	}
	}
	$pkginfo->configure(-state=>'disabled');
}

# user checker, alert the person running the app if not root
if($> ne "0"){
	$cuid = $main->DialogBox(-title=>"Effective User Id Error",-buttons=>["Ok"]);
	$cuid->add("Label",-text=>"Warning, you will not be able to install/uninstall applications. I must run from root or setuid!")->pack;
	$cuid->Show;
	$admin = 1;
	$pkgaddb->configure(-state=>'disabled');
	$pkgrmb->configure(-state=>'disabled');
	}else{
	$admin = 0;
}

sub pkglistr {
	opendir(DIR, "/var/log/packages") or warn ("Cannot open current directory! Autoload Aborted!");
	my @contents = readdir(DIR);
	closedir(DIR);
	# Links may be ignored completely:
	# No hidden files and ".." directories:
	# for we run setuid properly data must be untainted before Tk's use,was a pain to figure out

	$count = 0;
	while(@contents[$count] ne ""){
	@contents[$count] =~ /[\/\w\-._+]+/;             # includes 'file' chars + "\" and "+"
	$temp = $&;
	@contents[$count] = "$temp";
	$count++;
	}

	@contents = grep {!/^\./} @contents;
	@contents = sort { "\L$a" cmp "\L$b" } @contents; # sort, uses {to lower case} for comparison, to make case-insensitive
	$pkglist->delete(0,'end');
	foreach(@contents){
	$pkglist->insert(end,$_);
	}
}

MainLoop; #The main processing loop
